<?php
/**
 * Implements hook_field_formatter_info().
 */
function epub_field_formatter_info() {
  return array(
    'epub_cover' => array(
      'label' => t('EPUB cover image'),
      'description' => t('Displays the cover image of ebook'),
      'field types' => array('file'),
      'settings' => array(
        'width' => '100%',
      ),
    ),
    'epub_default' => array(
      'label' => t('EPUB default'),
      'description' => t('Displays ebook pages in an iframe'),
      'field types' => array('file'),
      'settings' => array(
        'width' => '100%',
        'height' => '100%',
      ),
    ),
    'epub_js' => array(
      'label' => t('Epub.js reader'),
      'description' => t('Displays ebook with epub.js'),
      'field types' => array('file'),
      'settings' => array(
        'width' => '100%',
        'height' => '800',
      ),
    ),
  );
}

function epub_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  $element = array();
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];
  switch ($display['type']) {
    case 'epub_cover':
      $element['width'] = array(
        '#type' => 'textfield',
        '#title' => t('Width'),
        '#default_value' => $settings['width'] ? $settings['width'] : '100%',
        '#description' => t('The width of ebook cover image.'),
      );
      return $element;
    case 'epub_default':
      $element['width'] = array(
        '#type' => 'textfield',
        '#title' => t('Width'),
        '#default_value' => $settings['width'] ? $settings['width'] : '100%',
        '#description' => t('The width of ebook viewer area.'),
      );
      $element['height'] = array(
        '#type' => 'textfield',
        '#title' => t('Height'),
        '#default_value' => $settings['height'] ? $settings['height'] : '100%',
        '#description' => t('The height of ebook viewer area.'),
      );
      return $element;
    case 'epub_js':
      $element['width'] = array(
        '#type' => 'textfield',
        '#title' => t('Width'),
        '#default_value' => $settings['width'] ? $settings['width'] : '100%',
        '#disabled' => TRUE,
        '#description' => t('The width of ebook viewer area. It\'s not in use.'),
      );
      $element['height'] = array(
        '#type' => 'textfield',
        '#title' => t('Height'),
        '#default_value' => $settings['height'] ? $settings['height'] : '800',
        '#description' => t('The height of ebook viewer area.'),
      );
      return $element;

  }
  return '';
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function epub_field_formatter_settings_summary($field, $instance, $view_mode) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];
  switch ($display['type']) {
    case 'epub_cover':
      return t('Width: @width', array('@width' => $settings['width'] ? $settings['width'] : '100%'));
    case 'epub_default':
      $args = array(
        '@width' => $settings['width'] ? $settings['width'] : '100%',
        '@height' => $settings['height'] ? $settings['height'] : '100%',
      );
      return t('Width: @width, Height: @height', $args);
    case 'epub_js':
      $args = array(
        '@width' => $settings['width'] ? $settings['width'] : '100%',
        '@height' => $settings['height'] ? $settings['height'] : '800',
      );
      return t('Width: @width, Height: @height', $args);
  }
  return '';
}

/**
 * Implements hook_field_formatter_view().
 */
function epub_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $element = array();
  $settings = $display['settings'];
  switch ($display['type']) {
    case 'epub_cover':
      foreach ($items as $delta => $item) {
        $element[$delta] = array(
          '#theme' => 'epub_formatter_cover',
          '#file' => (object)$item,
          '#width' => $settings['width'],
        );
      }
      break;
    case 'epub_default':
      foreach ($items as $delta => $item) {
        $element[$delta] = array(
          '#theme' => 'epub_formatter_default',
          '#file' => (object)$item,
          '#width' => $settings['width'],
          '#height' => $settings['height'],
        );
      }
      break;
    case 'epub_js':
      foreach ($items as $delta => $item) {
        $element[$delta] = array(
          '#theme' => 'epub_formatter_epubjs',
          '#file' => (object)$item,
          '#width' => $settings['width'],
          '#height' => $settings['height'],
        );
      }
      break;

  }
  return $element;
}

function theme_epub_formatter_cover($variables) {
  $file_url = file_create_url($variables['file']->uri);
  $filename = $variables['file']->filename;
  $fid = 'fid-' . $variables['file']->fid;

  $dir = 'public://epub_content/' . $variables['file']->fid;
  $file = drupal_realpath($variables['file']->uri);
  $output = t('<p>This eBook can\'t be opened.</p>');

  $scan = file_scan_directory($dir, '/.*\.opf$/');
  $opf = array_shift($scan);
  if (isset($opf)) {
    $opfXML = simplexml_load_file($opf->uri);
    $namespaces = $opfXML->getDocNamespaces();
    $opfXML->registerXPathNamespace('opf', 'http://www.idpf.org/2007/opf');
    $element = $opfXML->xpath('/opf:package/opf:metadata/opf:meta[@name="cover"]');
    if (empty($element)) {
        $variables = array(
          'path' => $GLOBALS['base_url'] . '/misc/help.png',
          'alt' => 'Book cover',
          'title' => $variables['file']->filename,
          'width' => $variables['width'],
          'attributes' => array(
            'class' => 'ebook-cover',
            'id' => $fid,
            'style' => 'border:1px solid black;',
           ),
        );

        $content = '<div class="ebook" id="' . $fid . '">' . l(theme('image', $variables), $file_url, $options = array('html'=>true)) . '</div>';
        $content .= '<p>' . t('A cover image is not required according to the ePub standard') . '</p>';
      }
      else {
      $attributes = $element[0]->attributes();
      $cover = epub_get_item($dir, (string)$attributes['content']);
      $variables = array(
        'path' => $cover,
        'alt' => 'Book cover',
        'title' => $variables['file']->filename,
        'width' => $variables['width'],
        'attributes' => array('
          class' => 'ebook-cover',
          'id' => $fid,
          'style' => 'border:1px solid black;',
        ),
      );
      //$content = l($filename, $file_url);
      $content = '<div class="ebook" id="' . $fid . '">' . l(theme('image', $variables), $file_url, $options = array('html'=>true)) . '</div>';
    }
  }

  return $content;
}

function theme_epub_formatter_default($variables) {
  $file_url = file_create_url($variables['file']->uri);
  $filename = $variables['file']->filename;
  $fid = 'fid-' . $variables['file']->fid;
  $download_link= t('Download: ') . theme('file_entity_file_link' , $variables);

  $dir = 'public://epub_content/' . $variables['file']->fid;
  $file = drupal_realpath($variables['file']->uri);
  $output = $download_link . t('<p>This eBook can\'t be opened.</p>');

  $scan = file_scan_directory($dir, '/.*\.opf$/');
  $opf = array_shift($scan);
  if (isset($opf)) {
    $opfXML = simplexml_load_file($opf->uri);
    $namespaces = $opfXML->getDocNamespaces();
    $opfXML->registerXPathNamespace('opf', 'http://www.idpf.org/2007/opf');
    $items = array();
    foreach ($opfXML->xpath('/opf:package/opf:spine/opf:itemref') as $key => $value) {
      $attributes = $value->attributes();
      $id = (string)$attributes['idref'];
      $items[$key] = file_create_url(epub_get_item($dir, $id));
    }
    $query = drupal_get_query_parameters();
    if (isset($query['page'])) {
      $current_page = $query['page'];
    }
    else {
      $current_page = 0;
    }
    $total = count($items);
    pager_default_initialize($total, 1);

    $content = array(
      '#type' => 'html_tag',
      '#tag' => 'iframe',
      '#value' => '',
      '#attributes' => array(
        'class' => array('ebook'),
        'id' => array($fid),
        'webkitallowfullscreen' => '',
        'mozallowfullscreen' => '',
        'allowfullscreen' => '',
        //'frameborder' => 'no',
        'width' => $variables['width'] ? $variables['width'] : '100%',
        'height' => $variables['height'] ? $variables['height'] : '100%',
        'src' => $items[$current_page],
      ),
    );
    $output =  $download_link . render($content) . theme('pager');
  }
  return $output;
}

function theme_epub_formatter_epubjs($variables) {
  //dpm($variables);
  $file_url = file_create_url($variables['file']->uri);
  $dir = file_create_url('public://epub_content/' . $variables['file']->fid);
  drupal_add_js(libraries_get_path('epub.js') . '/build/epub.min.js');
  drupal_add_js(libraries_get_path('epub.js') . '/build/hooks.min.js');
  drupal_add_js(libraries_get_path('epub.js') . '/build/reader.js');
  drupal_add_js(libraries_get_path('epub.js') . '/build/libs/screenfull.min.js');
//  drupal_add_js(libraries_get_path('epub.js') . '/hooks/extensions/pageturns.js');
//  drupal_add_js(libraries_get_path('epub.js') . '/build/libs/zip.min.js');
//  drupal_add_js(libraries_get_path('epub.js') . '/build/libs/inflate.js');

  drupal_add_js('"use strict"', array('type' => 'inline'));
  //drupal_add_js('var Book = ePub("'.$file_url.'", {width: '.$variables['width'].', height: '.$variables['width'].', spreads: false});', array('type' => 'inline'));
  drupal_add_js('var Book = ePub("'.$dir.'/", {height: '.$variables['height'].', spreads: false});', array('type' => 'inline'));
  return theme('epub_js', $variables);
}

